﻿Imports System.Drawing
Imports System.IO
Imports System.Globalization
Imports MindFusion.Diagramming
Imports MindFusion.Diagramming.Layout
Imports MindFusion.Diagramming.WebForms

Partial Public Class _Default
    Inherits System.Web.UI.Page

    
    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

        If Not IsPostBack Then

            diagramView.Behavior = Behavior.Modify

            diagramView.Diagram.DefaultShape = Shapes.Rectangle
            diagramView.Diagram.ContainersFoldable = False
            diagramView.Diagram.ContainerCaptionHeight = 6
            diagramView.Diagram.Bounds = New RectangleF(0, 0, 250, 195)
        End If

        scale = New Color() {Color.FromArgb(&HFF, &H20, &HC3, &H0), Color.FromArgb(&HFF, &HFF, &HFF, &H0), Color.FromArgb(&HFF, &HFF, &H80, &H0)}

        CreateTreeMap()
        AssignWeights()
        AssignColors()
        ArrangeDiagram()
    End Sub

    Private Sub CreateTreeMap()
        diagramView.Diagram.ClearAll()

        Dim doc = XDocument.Load(Server.MapPath("~/Data.xml"))
        For Each continentElement In doc.Element("continents").Elements("continent")
            Dim container = diagramView.Diagram.Factory.CreateContainerNode(0, 0, 10, 10)
            container.Tag = New Single() {Single.Parse(continentElement.Attribute("area").Value, CultureInfo.InvariantCulture), Single.Parse(continentElement.Attribute("population").Value, CultureInfo.InvariantCulture)}
            container.Caption = continentElement.Attribute("name").Value

            For Each countryElement In continentElement.Element("countries").Elements("country")
                Dim shape = diagramView.Diagram.Factory.CreateShapeNode(0, 0, 10, 10)
                shape.Tag = New Single() {Single.Parse(countryElement.Attribute("area").Value, CultureInfo.InvariantCulture), Single.Parse(countryElement.Attribute("population").Value, CultureInfo.InvariantCulture)}
                shape.Text = countryElement.Attribute("name").Value

                Dim area = Double.Parse(countryElement.Attribute("area").Value, CultureInfo.InvariantCulture)
                Dim population = Double.Parse(countryElement.Attribute("population").Value, CultureInfo.InvariantCulture)
                Dim areaQuantifier As String = "million sq. km."
                If area < 1 Then
                    area *= 1000
                    areaQuantifier = "thousand sq. km."
                End If
                Dim populationQuantifier As String = "billions"
                If population < 1 Then
                    population *= 1000
                    populationQuantifier = "millions"
                End If

                shape.ToolTip = String.Format("{0}{1}Area: ~{2} {3}{4}Population: ~{5} {6}.", shape.Text, "<br />", area, areaQuantifier, "<br />", _
                 population, populationQuantifier)
                container.Add(shape)
            Next
        Next
    End Sub


    Private Sub AssignWeights()
        For Each node In diagramView.Diagram.Nodes
            ' Ignore containers and container captions
            If TypeOf node Is ContainerNode OrElse node.Locked Then
                Continue For
            End If

            If TypeOf node Is ShapeNode Then
                Dim values = TryCast(node.Tag, Single())
                If values Is Nothing Then
                    Continue For
                End If

                If ddlWeight.SelectedValue = "Area" Then
                    node.Weight = values(0)
                End If
                If ddlWeight.SelectedValue = "Population" Then
                    node.Weight = values(1)
                End If
            End If
        Next
    End Sub

    Private Sub AssignColors()
        Dim largestArea = New Dictionary(Of ContainerNode, Double)()
        Dim largestPopulation = New Dictionary(Of ContainerNode, Double)()
        For Each node In diagramView.Diagram.Nodes
            If TypeOf node Is ContainerNode Then
                Dim container = DirectCast(node, ContainerNode)

                Dim area As Double = 0.0
                Dim population As Double = 0.0
                For Each child As DiagramNode In container.SubordinateGroup.AttachedNodes
                    Dim values = TryCast(child.Tag, Single())
                    If values Is Nothing Then
                        Continue For
                    End If

                    area = Math.Max(area, values(0))
                    population = Math.Max(population, values(1))
                Next

                largestArea(container) = area
                largestPopulation(container) = population
            End If
        Next

        For Each node In diagramView.Diagram.Nodes
            ' Ignore containers and container captions
            If TypeOf node Is ContainerNode OrElse node.Locked Then
                Continue For
            End If

            If TypeOf node Is ShapeNode Then
                Dim values = TryCast(node.Tag, Single())
                If values Is Nothing Then
                    Continue For
                End If

                Dim value As Double = 0
                Dim total As Double = 0
                If ddlColor.SelectedValue = "Area" Then
                    value = values(0)
                    total = largestArea(ContainerNode.GetContainer(node))
                End If
                If ddlColor.SelectedValue = "Population" Then
                    value = values(1)
                    total = largestPopulation(ContainerNode.GetContainer(node))
                End If
                node.Brush = New MindFusion.Drawing.SolidBrush(GetScaleColor(value, total))
            End If
        Next
    End Sub

    Private Function GetScaleColor(value As Double, total As Double) As Color
        Dim factor As Double = value / total
        If factor < 0 Then
            factor = 0
        End If
        If factor > 1 Then
            factor = 1
        End If

        Dim [step] As Double = 1.0 / (scale.Length - 1)
        Dim start As Color = scale(Math.Truncate(factor / [step]))
        Dim [end] As Color = scale(Math.Min(scale.Length - 1, Math.Truncate(factor / [step]) + 1))

        Dim factorBase As Double = factor - factor Mod [step]

        Return Color.FromArgb(&HFF, _
         CByte(CInt(start.R) + Math.Truncate((factor - factorBase) * (CInt([end].R) - CInt(start.R)) / [step])), _
         CByte(CInt(start.G) + Math.Truncate((factor - factorBase) * (CInt([end].G) - CInt(start.G)) / [step])), _
         CByte(CInt(start.B) + Math.Truncate((factor - factorBase) * (CInt([end].B) - CInt(start.B)) / [step])))

    End Function

    Private Sub ArrangeDiagram()
        Dim layout = New MindFusion.Diagramming.Layout.TreeMapLayout()
        layout.Padding = 0
        layout.ContainerPadding = 1
        layout.LayoutArea = New RectangleF(0, 0, 250, 195)
        layout.Arrange(diagramView.Diagram)

        ' Remove the text of smaller nodes
        For Each node In diagramView.Diagram.Nodes
            Dim shape = TryCast(node, ShapeNode)
            If shape Is Nothing OrElse shape.Locked Then
                Continue For
            End If

            If node.Bounds.Width < 10 OrElse node.Bounds.Height < 10 Then
                shape.Text = ""
            End If
        Next

        ' Resize caption nodes
        For Each node In diagramView.Diagram.Nodes
            If node.Locked AndAlso node.MasterGroup IsNot Nothing AndAlso node.MasterGroup.MainItem IsNot Nothing Then
                Dim bounds = node.Bounds
                bounds.Width = node.MasterGroup.MainItem.GetBounds().Width
                bounds.Height = Math.Min(30, node.MasterGroup.MainItem.GetBounds().Height)
                node.Bounds = bounds
            End If
        Next

        Zoom()
    End Sub

    Private Sub Zoom()
        diagramView.ZoomFactor = 100

        Dim rc As RectangleF = diagramView.DocToClient(diagramView.Diagram.GetContentBounds(False, True))

        Dim ctrlWidth As Single = diagramView.Width.Value
        Dim ctrlHeight As Single = diagramView.Height.Value
        Dim cntWidth As Single = rc.Width
        Dim cntHeight As Single = rc.Height

        If ctrlWidth > 0 AndAlso ctrlHeight > 0 AndAlso cntWidth > 0 AndAlso cntHeight > 0 Then
            Dim rx As Single = cntWidth / ctrlWidth
            Dim ry As Single = cntHeight / ctrlHeight
            Dim ratio As Single = Math.Max(rx, ry)

            Dim newZoom As Single = 100.0F / ratio
            If newZoom > 10 Then
                newZoom -= 2
            End If

            diagramView.ZoomFactor = newZoom
        End If
    End Sub

    Private scale As Color()
End Class